@@ -10,7 +10,7 @@ import numpy as np
 
 from debater_python_api.api.clients.key_point_summarization.KpsExceptions import KpsIllegalInputException
 from debater_python_api.api.clients.key_point_summarization.utils import read_dicts_from_df, create_dict_to_list, \
-    write_df_to_file, get_unique_sent_id, filter_dict_by_keys
+    write_df_to_file, get_unique_sent_id, filter_dict_by_keys, update_row_with_stance_data
 from debater_python_api.api.clients.key_point_summarization.docx_generator import save_hierarchical_graph_data_to_docx
 from debater_python_api.api.clients.key_point_summarization.graph_generator import create_graph_data, graph_data_to_hierarchical_graph_data, \
     get_hierarchical_graph_from_tree_and_subset_results, get_hierarchical_kps_data
@@ -30,13 +30,37 @@ class KpsResult:
         kps = self.get_key_points()
         if len(kps) == 0:
             logging.info("No key points found. Returning empty KpsResult.")
-        self.result_df = self._create_result_df()
         self.version = CURR_RESULTS_VERSION
         self.stances = set(result_json["job_metadata"]["per_stance"].keys()).difference({"no-stance"})
+        self.result_df = self._create_result_df()
         self.filter_min_relations = filter_min_relations
         self.kp_id_to_hierarchical_data = self._get_kp_id_to_hierarchical_data()
         self.summary_df = self._result_df_to_summary_df()
 
+    def get_unmapped_sentences_df(self):
+        """
+        return a df with all the sentences not matched during the job (either filtered for length/stance or no matching key point was found).
+        """
+        unmatched_sentences = self.result_json['unmatched_sentences']
+        sentences_data = self.result_json['sentences_data']
+        data_cols = ['span_start', 'span_end', 'num_tokens', 'argument_quality','sentence_text','stance']
+        rows = []
+
+        for s in unmatched_sentences:
+            comment_id = str(s["comment_id"])
+            sent_id_in_comment = str(s["sentence_id"])
+            comment_data = sentences_data[comment_id]
+            sent_data = comment_data["sentences"][sent_id_in_comment]
+            sent_row = [comment_id, sent_id_in_comment, comment_data["sents_in_comment"]] + [sent_data[c] for c in data_cols]
+            rows.append(sent_row)
+
+        cols = ['comment_id', 'sentence_id', "sents_in_comment"] + data_cols
+
+        unmatched_df = pd.DataFrame(rows, columns=cols)
+        unmatched_df = unmatched_df.rename(columns={"stance":"stance_dict"})
+        unmatched_df = unmatched_df.apply(lambda r: update_row_with_stance_data(r), axis=1)
+        return unmatched_df
+
     def get_domain(self):
         return self.result_json["job_metadata"]["general"]["domain"]
 
@@ -74,6 +98,7 @@ class KpsResult:
         kps_have_stance = False
         sentences_have_stance = False
         stance_keys = []
+        kps_to_stance = defaultdict(lambda: set())
         for keypoint_matching in self.result_json['keypoint_matchings']:
             kp = keypoint_matching['keypoint']
             if kp == "none":
@@ -106,6 +131,7 @@ class KpsResult:
                     sentences_have_stance = True
                 if kp_stance is not None:
                     match_row.append(kp_stance)
+                    kps_to_stance[kp].add(kp_stance)
                 matchings_rows.append(match_row)
         matchings_cols = ["kp", "sentence_text", "match_score", 'comment_id', 'sentence_id', 'sents_in_comment',
                           'span_start', 'span_end', 'num_tokens', 'argument_quality', 'kp_quality', "sent_kp_quality"]
@@ -113,7 +139,16 @@ class KpsResult:
             matchings_cols.extend([f'{k}_score' for k in stance_keys] + ["selected_stance", "stance_conf"])
         if kps_have_stance:
             matchings_cols.append('kp_stance')
-        return pd.DataFrame(matchings_rows, columns=matchings_cols)
+        results_df = pd.DataFrame(matchings_rows, columns=matchings_cols)
+
+        if len(self.stances) > 1:
+            kp_with_two_stances = list(filter(lambda x: len(kps_to_stance[x]) > 1, kps_to_stance.keys()))
+            if len(kp_with_two_stances) > 0:
+                results_df.loc[:,"kp"] = results_df.apply(lambda r: self._get_kp_stance_str(r['kp'], r['kp_stance']) if r['kp'] in kp_with_two_stances else r['kp'], axis = 1)
+        return results_df
+
+    def _get_kp_stance_str(self, kp, stance):
+        return f"{kp} ({stance})"
 
     def _result_df_to_summary_df(self):
 
@@ -123,9 +158,8 @@ class KpsResult:
 
         kp_to_dicts = create_dict_to_list([(d['kp'], d) for d in dicts])
 
-        general_metadata = self.result_json["job_metadata"]["general"]
-        n_total_sentences = general_metadata["n_sentences"]
-        n_total_comments = general_metadata["n_comments"]
+        n_total_sentences = self._get_number_of_unique_sentences()
+        n_total_comments = self._get_number_of_unique_comments()
 
         summary_rows = []
         summary_cols = ["key_point", '#comments', 'comments_coverage', "#sentences", 'sentences_coverage', "stance","kp_id","parent_id", "n_comments_subtree"]
@@ -185,7 +219,7 @@ class KpsResult:
     @staticmethod
     def create_from_result_json(result_json, filter_min_relations=0.4):
         """
-        Create KpsResults from results_json
+        Create KpsResults from result_json
         :param result_json: the json object obtained from the client via "get_result" or "get_result_from_futures"
         :param filter_min_relations: minimal relation score between key points to be considered related in the hierarchy
         :return: KpsResult object
@@ -284,12 +318,15 @@ class KpsResult:
         :param result_name: name of the results to appear in the output files.
         :param n_matches_in_docx: optional, number of top matches to write in the textual summary (docx file). Pass None for all matches.
         :param include_match_score_in_docx: optional, when set to true, the match score between the sentence and the key point is added.
-        Creates 3 outout files:
+        Creates 4 outout files:
+             * <ouput_dir>/<result_name>.json : this results as a json file.
              * <ouput_dir>/<result_name>.csv : full results as csv .
              * <ouput_dir>/<result_name>_kps_summary.csv : summary results as csv.
              *  <result_file>_hierarchical.docx: This Microsoft Word document shows the key point hierarchy and matching sentences
             as a user-friendly report.
         """
+        json_results_name = os.path.join(output_dir, result_name+".json")
+        self.save(json_results_name)
 
         result_file = os.path.join(output_dir, result_name+".csv")
         write_df_to_file(self.result_df, result_file)
@@ -370,24 +407,22 @@ class KpsResult:
                 cov_from_stance = float(n_matched_comments) / float(n_comments_with_stance) * 100.0
             print(f'Coverage ({stance_str} comments): %.2f' % cov_from_stance)
 
-        kp_to_n_comments = self._get_kp_to_n_matched_comments()
-        if "none" in kp_to_n_comments:
-            del kp_to_n_comments["none"]
+        kp_stance_to_n_comments = self._get_kp_stance_to_n_matched_comments()
 
-        sorted_kps = list(sorted(kp_to_n_comments.keys(), key=lambda x: kp_to_n_comments[x], reverse=True))
-        total_n_kps = len(sorted_kps)
+        sorted_kps_stance = list(sorted(kp_stance_to_n_comments.keys(), key=lambda x: kp_stance_to_n_comments[x], reverse=True))
+        total_n_kps = len(sorted_kps_stance)
         n_top_kps = n_top_kps if n_top_kps else total_n_kps
         n_displayed_kps = np.min([n_top_kps, total_n_kps])
-        sorted_kps = sorted_kps[:n_displayed_kps]
+        sorted_kps = sorted_kps_stance[:n_displayed_kps]
         print(f'Displaying {n_displayed_kps} key points out of {total_n_kps}:')
 
-        kp_to_matching = {keypoint_matching['keypoint']:  keypoint_matching for keypoint_matching in keypoint_matchings}
+        kp_stance_to_matching = {(keypoint_matching['keypoint'],keypoint_matching.get('stance')):  keypoint_matching for keypoint_matching in keypoint_matchings}
 
-        for kp in sorted_kps:
-            keypoint_matching = kp_to_matching[kp]
-            stance = None if 'stance' not in keypoint_matching else keypoint_matching['stance']
-            n_comments = kp_to_n_comments[kp]
-            print_kp(kp, stance, keypoint_matching['matching'], sentences_data, n_sentences_per_kp, n_comments)
+        for kp_stance in sorted_kps:
+            keypoint_matching = kp_stance_to_matching[kp_stance]
+            #stance = None if 'stance' not in keypoint_matching else keypoint_matching['stance']
+            n_comments = kp_stance_to_n_comments[kp_stance]
+            print_kp(kp_stance[0],kp_stance[1], keypoint_matching['matching'], sentences_data, n_sentences_per_kp, n_comments)
 
     def _get_number_of_unique_sentences(self, include_unmatched=True):
         if include_unmatched:
@@ -400,7 +435,7 @@ class KpsResult:
                     total_sentences = total_sentences.union(matching_sents_ids)
             return len(total_sentences)
 
-    def _get_comparison_df(self, results_to_kp_to_n_comments, results_to_total_comments, titles, kp_to_stance):
+    def _get_comparison_df(self, results_to_kp_to_n_comments, results_to_total_comments, titles):
         ordered_kps = []
         cols = ['key point', 'stance']
 
@@ -411,9 +446,6 @@ class KpsResult:
             ordered_kps += new_kps
             cols.extend([f"{title}_n_comments", f"{title}_percent"])
 
-        if "none" in ordered_kps:
-            ordered_kps.remove("none")
-
         add_change = False
         if len(titles) == 2:
             cols.append("change_percent")
@@ -422,11 +454,11 @@ class KpsResult:
         rows = []
         title_to_precent = {}
         total_row = ["total", ""]
-        for i, kp in enumerate(ordered_kps):
-            row = [kp, kp_to_stance[kp]]
+        for i, kp_stance in enumerate(ordered_kps):
+            row = [kp_stance[0], kp_stance[1]]
 
             for title in titles:
-                n_comments_title_kp = results_to_kp_to_n_comments[title].get(kp, 0)
+                n_comments_title_kp = results_to_kp_to_n_comments[title].get(kp_stance, 0)
                 percent_comments_title_kp = (
                             100 * n_comments_title_kp / results_to_total_comments[title]) if n_comments_title_kp else 0
                 row.extend([n_comments_title_kp, f'{percent_comments_title_kp:.2f}%'])
@@ -440,10 +472,11 @@ class KpsResult:
                     total_row.append("")
 
             rows.append(row)
-        rows.append(total_row)
+        if len(rows) > 0:
+            rows.append(total_row)
         comparison_df = pd.DataFrame(rows, columns=cols)
 
-        if len(set(kp_to_stance.values()).difference({"no-stance", None})) == 0:
+        if (set(comparison_df["stance"]) == {None}):
             comparison_df = comparison_df[[c for c in comparison_df.columns if c != "stance"]]
         return comparison_df
 
@@ -462,12 +495,11 @@ class KpsResult:
 
         titles = (["full"] if include_full else []) + sorted(comments_subsets_dict.keys(), key=lambda x: results_to_total_comments[x], reverse=True)
 
-        results_to_kp_to_n_comments = {"full": self._get_kp_to_n_matched_comments()} if include_full else {}
+        results_to_kp_to_n_comments = {"full": self._get_kp_stance_to_n_matched_comments()} if include_full else {}
         results_to_kp_to_n_comments.update(
-            {title: self._get_kp_to_n_matched_comments(comments_subset=comment_ids) for title, comment_ids in comments_subsets_dict.items()})
+            {title: self._get_kp_stance_to_n_matched_comments(comments_subset=comment_ids) for title, comment_ids in comments_subsets_dict.items()})
 
-        kp_to_stance = self.get_kp_to_stance()
-        return self._get_comparison_df(results_to_kp_to_n_comments, results_to_total_comments, titles, kp_to_stance)
+        return self._get_comparison_df(results_to_kp_to_n_comments, results_to_total_comments, titles)
 
     def compare_with_other_results(self, this_title : str, other_results_dict : Dict[str,'KpsResult']):
         """
@@ -481,14 +513,10 @@ class KpsResult:
         results_to_total_comments.update({title:result._get_number_of_unique_comments() for title,result in other_results_dict.items()})
         titles = [this_title] + sorted(other_results_dict.keys(), key=lambda x:results_to_total_comments[x], reverse=True)
 
-        results_to_kp_to_n_comments = {this_title:self._get_kp_to_n_matched_comments()}
-        results_to_kp_to_n_comments.update({title:result._get_kp_to_n_matched_comments()  for title,result in other_results_dict.items()})
+        results_to_kp_to_n_comments = {this_title:self._get_kp_stance_to_n_matched_comments()}
+        results_to_kp_to_n_comments.update({title:result._get_kp_stance_to_n_matched_comments()  for title,result in other_results_dict.items()})
 
-        kp_to_stance = self.get_kp_to_stance()
-        for result in other_results_dict.values():
-            kp_to_stance.update(result.get_kp_to_stance())
-
-        return self._get_comparison_df(results_to_kp_to_n_comments, results_to_total_comments, titles, kp_to_stance)
+        return self._get_comparison_df(results_to_kp_to_n_comments, results_to_total_comments, titles)
 
     @staticmethod
     def get_merged_pro_con_results(pro_result: 'KpsResult', con_result: 'KpsResult'):
@@ -517,7 +545,9 @@ class KpsResult:
                     none_matchings.extend(keypoint_matching["matching"])
                 else:
                     keypoint_matchings.append(keypoint_matching)
-        keypoint_matchings.append({'keypoint':'none', 'matching':none_matchings})
+
+        if len(none_matchings) > 0: # if there were non keypoints in original results, for backwards compatibility
+            keypoint_matchings.append({'keypoint':'none', 'matching':none_matchings})
         keypoint_matchings.sort(key=lambda matchings: len(matchings['matching']), reverse=True)
 
         sentences_data = pro_result.result_json['sentences_data']
@@ -527,16 +557,57 @@ class KpsResult:
             for sent_id_in_comment, sent_data in comment_data_dict["sentences"].items():
                  sentences_data[comment_id]["sentences"][sent_id_in_comment] = sent_data
 
+        pro_unmatched_sents = set([tuple(d.items()) for d in pro_result.result_json["unmatched_sentences"]])
+        con_unmatched_sents = set([tuple(d.items()) for d in con_result.result_json["unmatched_sentences"]])
+        unmatched_sents = pro_unmatched_sents.intersection(con_unmatched_sents)
+        unmatched_sents = [dict(s_data) for s_data in unmatched_sents]
+
         pro_metadata = pro_result.result_json["job_metadata"]
         con_metadata = con_result.result_json["job_metadata"]
         new_metadata = {"general":pro_metadata["general"],
                         "per_stance":{"pro":pro_metadata["per_stance"]["pro"], "con":con_metadata["per_stance"]["con"]}}
         combined_results_json = {'keypoint_matchings':keypoint_matchings, "sentences_data":sentences_data,
                                  "version":CURR_RESULTS_VERSION,
-                                 "job_metadata":new_metadata}
+                                 "job_metadata":new_metadata, "unmatched_sentences":unmatched_sents}
         filter_min_relations = np.min([pro_result.filter_min_relations, con_result.filter_min_relations])
         return KpsResult.create_from_result_json(combined_results_json, filter_min_relations=filter_min_relations)
 
+    def get_single_stance_from_merged_results(self, stance):
+        stances = list(self.get_stance_to_job_id().keys())
+        if len(stances) == 1:
+            if stances[0] == stance:
+                logging.info("Results already contain the requested stance only, retuning self")
+                return self
+
+        assert stance in stances, f"Requested stance ({stance}) not in results  stance: ({stances})."
+
+        merged_json = self.result_json
+        new_metadata = {"general": merged_json["job_metadata"]["general"],
+                        "per_stance": {stance: merged_json["job_metadata"]["per_stance"][stance]}
+                        }
+
+        new_keypoint_matchings = []
+        new_unmatched = merged_json["unmatched_sentences"].copy()
+        for keypoint_matching in merged_json["keypoint_matchings"]:
+            if keypoint_matching["stance"] == stance:
+                new_keypoint_matchings.append(keypoint_matching)
+            else:
+                for matching in keypoint_matching["matching"]:
+                    unmatched = {"comment_id": matching["comment_id"],
+                                 "sentence_id": matching["sentence_id"]}
+                    if unmatched not in new_unmatched:
+                        new_unmatched.append(unmatched)
+
+        new_results_json = {
+                    "keypoint_matchings": new_keypoint_matchings,
+                    "sentences_data": merged_json["sentences_data"].copy(),
+                    "unmatched_sentences": list(new_unmatched),
+                    "version": merged_json["version"],
+                    "job_metadata": new_metadata
+                   }
+
+        return KpsResult.create_from_result_json(new_results_json, filter_min_relations=self.filter_min_relations)
+
     @staticmethod
     # If result_json is of the old server version, convert to version 2.0
     # 1.0 - received from the server, must have only one stance (merging pro_con is only on v2)
@@ -544,6 +615,13 @@ class KpsResult:
         metadata = result_json["job_metadata"]
         kps_stance = KpsResult._get_stance_from_server_stances(metadata["stances"])
         sentences_data = {}
+
+        unmapped_sentences = result_json.get("unused_sentences", []) # for server backward compatibility
+        unmapped_sentences_ids = []
+        for s in unmapped_sentences:
+            KpsResult._add_sentence_to_sentences_data(s, sentences_data)
+            unmapped_sentences_ids.append({"comment_id":s["comment_id"], "sentence_id":str(s["sentence_id"])})
+
         new_matchings = []
         for keypoint_matching in result_json['keypoint_matchings']:
             kp = keypoint_matching['keypoint']
@@ -553,36 +631,45 @@ class KpsResult:
                 new_kp_matching["stance"] = kps_stance
 
             for match in keypoint_matching['matching']:
+
+                if kp == "none":
+                    unmapped_sentences_ids.append({"comment_id": match["comment_id"], "sentence_id": str(match["sentence_id"])})
+
                 if "kp_quality" not in new_kp_matching:
                     kpq = match.get("kp_quality", 0)
                     new_kp_matching["kp_quality"] = kpq
 
-                comment_id = match["comment_id"]
-                sent_id_in_comment = str(match["sentence_id"])
+                KpsResult._add_sentence_to_sentences_data(match, sentences_data)
 
                 new_kp_matching["matching"].append(
-                    {"comment_id": comment_id, "sentence_id": str(sent_id_in_comment), "score": match["score"]})
-                if comment_id not in sentences_data:
-                    sentences_data[comment_id] = {"sents_in_comment": match["sents_in_comment"], "sentences": {}}
+                    {"comment_id": match["comment_id"], "sentence_id": str(match["sentence_id"]), "score": match["score"]})
 
-                if sent_id_in_comment not in sentences_data[comment_id]["sentences"]:
-                    sent_data = {c: match[c] for c in
-                                 ["sentence_text", "span_start", "span_end", "num_tokens", "argument_quality","kp_quality"]}
-                    if "stance" in match:
-                        sent_data["stance"] = dict(match["stance"])
-                    sentences_data[comment_id]["sentences"][sent_id_in_comment] = sent_data.copy()
-
-            new_matchings.append(new_kp_matching)
+            if kp != 'none':
+                new_matchings.append(new_kp_matching)
 
         per_stance_dict = {kps_stance: filter_dict_by_keys(metadata, ['description', 'run_params', 'job_id',
                                                                       'n_sentences_stance', 'n_comments_stance'])}
 
         metadata_v2 = {"general": filter_dict_by_keys(metadata, ['domain', 'user_id', 'n_sentences',
                                                                  'n_sentences_unfiltered', 'n_comments',
-                                                                 'n_comments_unfiltered']),
+                                                                 'n_comments_unfiltered','domain_params']),
                        "per_stance": per_stance_dict}
         return {"keypoint_matchings": new_matchings, "sentences_data": sentences_data, "version": CURR_RESULTS_VERSION,
-                "job_metadata": metadata_v2}
+                "job_metadata": metadata_v2, "unmatched_sentences":unmapped_sentences_ids}
+
+    @staticmethod
+    def _add_sentence_to_sentences_data(match, sentences_data):
+        comment_id = match["comment_id"]
+        sent_id_in_comment = str(match["sentence_id"])
+        if comment_id not in sentences_data:
+            sentences_data[comment_id] = {"sents_in_comment": match["sents_in_comment"], "sentences": {}}
+
+        if sent_id_in_comment not in sentences_data[comment_id]["sentences"]:
+            sent_data = {c: match.get(c) for c in
+                         ["sentence_text", "span_start", "span_end", "num_tokens", "argument_quality", "kp_quality"]}
+            if "stance" in match:
+                sent_data["stance"] = dict(match["stance"])
+            sentences_data[comment_id]["sentences"][sent_id_in_comment] = sent_data.copy()
 
     def _get_number_of_unique_comments(self, include_unmatched=True):
         if include_unmatched:
@@ -595,18 +682,20 @@ class KpsResult:
                     total_comments = total_comments.union(matching_sents_ids)
             return len(total_comments)
 
-    def _get_kp_to_n_matched_comments(self, comments_subset = None):
-        kp_n_comments = {}
+    def _get_kp_stance_to_n_matched_comments(self, comments_subset = None):
+        kp_stance_n_comments = {}
         for keypoint_matching in self.result_json['keypoint_matchings']:
             kp = keypoint_matching["keypoint"]
+            kp_stance = keypoint_matching.get("stance")
             matching_comments_set = set(match["comment_id"] for match in keypoint_matching["matching"])
 
             if comments_subset:
                 matching_comments_set = set(filter(lambda x:x in comments_subset, matching_comments_set))
 
             n_matches = len(matching_comments_set)
-            kp_n_comments[kp] = n_matches
-        return kp_n_comments
+            kp_stance_n_comments[(kp,kp_stance)] = n_matches
+
+        return kp_stance_n_comments
 
     def get_key_points(self):
         return [keypoint_matching["keypoint"] for keypoint_matching in self.result_json['keypoint_matchings'] if keypoint_matching["keypoint"] != "none"]
@@ -635,8 +724,8 @@ class KpsResult:
         No hierarchy is generated in this setting.
         :return: new KpsResult after choosing the top kp for each sentence.
         """
-        new_results_df = self.result_df.sort_values(by=["sentence_text","match_score"], ascending=[True, False])
-        top_preds_df = pd.concat([group.head(1) for _,group in new_results_df.groupby(by="sentence_text")])
+        new_results_df = self.result_df.sort_values(by=["match_score"], ascending=[False])
+        top_preds_df = pd.concat([group.head(1) for _,group in new_results_df.groupby(by=["comment_id","sentence_id"])])
 
         kp_to_args = list(zip(top_preds_df["kp"],zip(top_preds_df["comment_id"], top_preds_df["sentence_id"])))
         kp_to_args = create_dict_to_list(kp_to_args)
@@ -648,13 +737,20 @@ class KpsResult:
             new_keypoint_matching = keypoint_matching.copy()
             kp = new_keypoint_matching["keypoint"]
             if kp != "none":
-                new_keypoint_matching["matching"] = list(filter(lambda x: (x["comment_id"],int(x["sentence_id"])) in kp_to_args[kp], new_keypoint_matching["matching"]))
+                kp_stance = new_keypoint_matching.get('stance')
+                arg_for_kp = kp_to_args.get(kp)
+                if arg_for_kp is None:
+                    kp_stance = self._get_kp_stance_str(kp, kp_stance)
+                    arg_for_kp = kp_to_args.get(kp_stance)
+                if arg_for_kp:
+                    new_keypoint_matching["matching"] = list(filter(lambda x: (x["comment_id"],int(x["sentence_id"])) in arg_for_kp, new_keypoint_matching["matching"]))
             if len(new_keypoint_matching["matching"]) > 0:
                 new_keypoint_matchings.append(new_keypoint_matching)
         new_keypoint_matchings.sort(key=lambda x: len(x["matching"]), reverse=True)
 
         new_json = {"sentences_data": self.result_json["sentences_data"].copy(), "version": self.result_json["version"],
-                    "job_metadata":new_meta_data, "keypoint_matchings":new_keypoint_matchings}
+                    "job_metadata":new_meta_data, "keypoint_matchings":new_keypoint_matchings,
+                    "unmatched_sentences": self.result_json["unmatched_sentences"]}
 
         return KpsResult.create_from_result_json(new_json)
 
@@ -664,7 +760,7 @@ class KpsResult:
         elif len(self.stances) == 1:
             return list(self.stances)[0]
         return "pro and con"
-
-    def get_kp_to_stance(self):
-         keypoint_matchings = self.result_json["keypoint_matchings"]
-         return {keypoint_matching['keypoint']:keypoint_matching.get("stance", None) for keypoint_matching in keypoint_matchings}
+    #
+    # def get_kp_to_stance(self):
+    #      keypoint_matchings = self.result_json["keypoint_matchings"]
+    #      return {keypoint_matching['keypoint']:keypoint_matching.get("stance", None) for keypoint_matching in keypoint_matchings}
